#ifndef __CLICKESTIMATOR_HH
#define __CLICKESTIMATOR_HH

#include <QtCore/QObject>
#include <boost/smart_ptr.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <deque>

class ClickEstimator: public QObject
{
  Q_OBJECT
public:
  typedef boost::numeric::ublas::vector< double > Vector;
  ClickEstimator( QObject *parent = 0 );
  void setPosition( const Vector &v );
  void clearPosition(void);
  int getDelay(void) { return delay; }
  int getSensitivity(void) { return sensitivity; }
public slots:
  void setDelay(int);
  void setSensitivity(int);
  void setDragging(bool);
signals:
  void mouseMove(int,int);
  void varianceChanged(int);
  void click();
  void startDrag();
  void stopDrag();
protected:
  Vector average(void);
  int delay;
  int sensitivity;
  bool stopped;
  bool dragging;
  bool clicked;
  std::deque< Vector > positions;
};

typedef boost::shared_ptr< ClickEstimator > ClickEstimatorPtr;

#endif
